@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-only
 /*
  * High resolution timer test software
  *
@@ -6,10 +7,6 @@
  * (C) 2008-2012 Clark Williams <williams@redhat.com>
  * (C) 2005-2007 Thomas Gleixner <tglx@linutronix.de>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License Version
- * 2 as published by the Free Software Foundation.
- *
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,6 +36,7 @@
 #include "rt_numa.h"
 
 #include "rt-utils.h"
+#include "rt-numa.h"
 
 #include <bionic.h>
 
@@ -52,8 +50,6 @@
 #define SCHED_NORMAL SCHED_OTHER
 #endif
 
-#define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
-
 #define sigev_notify_thread_id _sigev_un._tid
 
 #ifdef __UCLIBC__
@@ -67,12 +63,12 @@ static int clock_nanosleep(clockid_t clock_id, int flags, const struct timespec
 	if (clock_id == CLOCK_THREAD_CPUTIME_ID)
 		return -EINVAL;
 	if (clock_id == CLOCK_PROCESS_CPUTIME_ID)
-		clock_id = MAKE_PROCESS_CPUCLOCK (0, CPUCLOCK_SCHED);
+		clock_id = MAKE_PROCESS_CPUCLOCK(0, CPUCLOCK_SCHED);
 
 	return syscall(__NR_clock_nanosleep, clock_id, flags, req, rem);
 }
 
-int sched_setaffinity (__pid_t __pid, size_t __cpusetsize,
+int sched_setaffinity(__pid_t __pid, size_t __cpusetsize,
 		       __const cpu_set_t *__cpuset)
 {
 	return -EINVAL;
@@ -89,9 +85,6 @@ extern int clock_nanosleep(clockid_t __clock_id, int __flags,
 			   struct timespec *__rem);
 #endif	/* __UCLIBC__ */
 
-#define USEC_PER_SEC		1000000
-#define NSEC_PER_SEC		1000000000
-
 #define HIST_MAX		1000000
 
 #define MODE_CYCLIC		0
@@ -184,6 +177,7 @@ static void trigger_update(struct thread_param *par, int diff, int64_t ts);
 
 static int shutdown;
 static int tracelimit = 0;
+static int trace_marker = 0;
 static int verbose = 0;
 static int oscope_reduction = 1;
 static int lockall = 0;
@@ -221,8 +215,6 @@ static pthread_barrier_t align_barr;
 static pthread_barrier_t globalt_barr;
 static struct timespec globalt;
 
-static char *procfileprefix = "/proc/sys/kernel/";
-static char *fileprefix;
 static char fifopath[MAX_PATH];
 static char histfile[MAX_PATH];
 
@@ -230,10 +222,18 @@ static struct thread_param **parameters;
 static struct thread_stat **statistics;
 
 static void print_stat(FILE *fp, struct thread_param *par, int index, int verbose, int quiet);
+static void rstat_print_stat(struct thread_param *par, int index, int verbose, int quiet);
+static void rstat_setup(void);
 
 static int latency_target_fd = -1;
 static int32_t latency_target_value = 0;
 
+static int rstat_ftruncate(int fd, off_t len);
+static int rstat_fd = -1;
+/* strlen("/cyclictest") + digits in max pid len + '\0' */
+#define SHM_BUF_SIZE 19
+static char shm_name[SHM_BUF_SIZE];
+
 /* Latency trick
  * if the file /dev/cpu_dma_latency exists,
  * open it and write a zero into it. This will tell
@@ -284,133 +284,13 @@ enum {
 	ERROR_NOTFOUND	= -2,
 };
 
-static int trace_fd     = -1;
-static int tracemark_fd = -1;
-
-static inline void tsnorm(struct timespec *ts)
-{
-	while (ts->tv_nsec >= NSEC_PER_SEC) {
-		ts->tv_nsec -= NSEC_PER_SEC;
-		ts->tv_sec++;
-	}
-}
-
-static inline int tsgreater(struct timespec *a, struct timespec *b)
-{
-	return ((a->tv_sec > b->tv_sec) ||
-		(a->tv_sec == b->tv_sec && a->tv_nsec > b->tv_nsec));
-}
-
-static inline int64_t calcdiff(struct timespec t1, struct timespec t2)
-{
-	int64_t diff = USEC_PER_SEC * (long long)((int) t1.tv_sec - (int) t2.tv_sec);
-	diff += ((int) t1.tv_nsec - (int) t2.tv_nsec) / 1000;
-	return diff;
-}
-
-static inline int64_t calcdiff_ns(struct timespec t1, struct timespec t2)
-{
-	int64_t diff;
-	diff = NSEC_PER_SEC * (int64_t)((int) t1.tv_sec - (int) t2.tv_sec);
-	diff += ((int) t1.tv_nsec - (int) t2.tv_nsec);
-	return diff;
-}
-
-static inline int64_t calctime(struct timespec t)
-{
-	int64_t time;
-	time = USEC_PER_SEC * t.tv_sec;
-	time += ((int) t.tv_nsec) / 1000;
-	return time;
-}
-
-static int trace_file_exists(char *name)
-{
-       struct stat sbuf;
-       char *tracing_prefix = get_debugfileprefix();
-       char path[MAX_PATH];
-       strcat(strcpy(path, tracing_prefix), name);
-       return stat(path, &sbuf) ? 0 : 1;
-}
-
-#define TRACEBUFSIZ 1024
-static __thread char tracebuf[TRACEBUFSIZ];
-
-static void tracemark(char *fmt, ...) __attribute__((format(printf, 1, 2)));
-static void tracemark(char *fmt, ...)
-{
-	va_list ap;
-	int len;
-
-	/* bail out if we're not tracing */
-	/* or if the kernel doesn't support trace_mark */
-	if (tracemark_fd < 0 || trace_fd < 0)
-		return;
-
-	va_start(ap, fmt);
-	len = vsnprintf(tracebuf, TRACEBUFSIZ, fmt, ap);
-	va_end(ap);
-
-	/* write the tracemark message */
-	write(tracemark_fd, tracebuf, len);
-
-	/* now stop any trace */
-	write(trace_fd, "0\n", 2);
-}
-
-static void open_tracemark_fd(void)
-{
-	char path[MAX_PATH];
-
-	/*
-	 * open the tracemark file if it's not already open
-	 */
-	if (tracemark_fd < 0) {
-		sprintf(path, "%s/%s", fileprefix, "trace_marker");
-		tracemark_fd = open(path, O_WRONLY);
-		if (tracemark_fd < 0) {
-			warn("unable to open trace_marker file: %s\n", path);
-			return;
-		}
-	}
-
-	/*
-	 * if we're not tracing and the tracing_on fd is not open,
-	 * open the tracing_on file so that we can stop the trace
-	 * if we hit a breaktrace threshold
-	 */
-	if (trace_fd < 0) {
-		sprintf(path, "%s/%s", fileprefix, "tracing_on");
-		if ((trace_fd = open(path, O_WRONLY)) < 0)
-			warn("unable to open tracing_on file: %s\n", path);
-	}
-}
-
-static void debugfs_prepare(void)
-{
-	if (mount_debugfs(NULL))
-		fatal("could not mount debugfs");
-
-	fileprefix = get_debugfileprefix();
-	if (!trace_file_exists("tracing_enabled") &&
-	    !trace_file_exists("tracing_on"))
-		warn("tracing_enabled or tracing_on not found\n"
-		     "debug fs not mounted");
-}
-
-static void enable_trace_mark(void)
-{
-	debugfs_prepare();
-	open_tracemark_fd();
-}
-
 /*
  * Raise the soft priority limit up to prio, if that is less than or equal
  * to the hard limit
  * if a call fails, return the error
  * if successful return 0
  * if fails, return -1
-*/
+ */
 static int raise_soft_prio(int policy, const struct sched_param *param)
 {
 	int err;
@@ -470,7 +350,8 @@ try_again:
 		if (err == EPERM) {
 			int err1;
 			err1 = raise_soft_prio(policy, param);
-			if (!err1) goto try_again;
+			if (!err1)
+				goto try_again;
 		}
 	}
 
@@ -641,7 +522,7 @@ static void *timerthread(void *param)
 		CPU_ZERO(&mask);
 		CPU_SET(par->cpu, &mask);
 		thread = pthread_self();
-		if (pthread_setaffinity_np(thread, sizeof(mask), &mask) == -1)
+		if (pthread_setaffinity_np(thread, sizeof(mask), &mask) != 0)
 			warn("Could not set CPU affinity to CPU #%d\n",
 			     par->cpu);
 	}
@@ -669,7 +550,7 @@ static void *timerthread(void *param)
 		fatal("timerthread%d: failed to set priority to %d\n",
 		      par->cpu, par->prio);
 
-	if(smi) {
+	if (smi) {
 		par->msr_fd = open_msr_file(par->cpu);
 		if (par->msr_fd < 0)
 			fatal("Could not open MSR interface, errno: %d\n",
@@ -687,7 +568,7 @@ static void *timerthread(void *param)
 			clock_gettime(par->clock, &globalt);
 			if (secaligned) {
 				/* Ensure that the thread start timestamp is not
-				   in the past */
+				 * in the past */
 				if (globalt.tv_nsec > 900000000)
 					globalt.tv_sec += 2;
 				else
@@ -798,7 +679,7 @@ static void *timerthread(void *param)
 		ret = clock_gettime(par->clock, &now);
 		if (ret != 0) {
 			if (ret != EINTR)
-				warn("clock_getttime() failed. errno: %d\n",
+				warn("clock_gettime() failed. errno: %d\n",
 				     errno);
 			goto out;
 		}
@@ -827,23 +708,22 @@ static void *timerthread(void *param)
 		}
 		stat->avg += (double) diff;
 
-		if (trigger && (diff > trigger)) {
+		if (trigger && (diff > trigger))
 			trigger_update(par, diff, calctime(now));
-		}
-
 
 		if (duration && (calcdiff(now, stop) >= 0))
 			shutdown++;
 
 		if (!stopped && tracelimit && (diff > tracelimit)) {
 			stopped++;
-			tracemark("hit latency threshold (%llu > %d)",
-				  (unsigned long long) diff, tracelimit);
 			shutdown++;
 			pthread_mutex_lock(&break_thread_id_lock);
-			if (break_thread_id == 0)
+			if (break_thread_id == 0) {
 				break_thread_id = stat->tid;
-			break_thread_value = diff;
+				tracemark("hit latency threshold (%llu > %d)",
+					  (unsigned long long) diff, tracelimit);
+				break_thread_value = diff;
+			}
 			pthread_mutex_unlock(&break_thread_id_lock);
 		}
 		stat->act = diff;
@@ -949,7 +829,7 @@ static void display_help(int error)
 	       "	 --latency=PM_QOS  write PM_QOS to /dev/cpu_dma_latency\n"
 	       "-F       --fifo=<path>     create a named pipe at path and write stats to it\n"
 	       "-h       --histogram=US    dump a latency histogram to stdout after the run\n"
-	       "                           US is the max latency time to be be tracked in microseconds\n"
+	       "                           US is the max latency time to be tracked in microseconds\n"
 	       "			   This option runs all threads at the same priority.\n"
 	       "-H       --histofall=US    same as -h except with an additional summary column\n"
 	       "	 --histfile=<path> dump the latency histogram to <path> instead of stdout\n"
@@ -960,7 +840,7 @@ static void display_help(int error)
 	       "			   but will not drain your battery so quickly\n"
 	       "-m       --mlockall        lock current and future memory allocations\n"
 	       "-M       --refresh_on_max  delay updating the screen until a new max\n"
-	       "			   latency is hit. Userful for low bandwidth.\n"
+	       "			   latency is hit. Useful for low bandwidth.\n"
 	       "-N       --nsecs           print results in ns instead of us (default us)\n"
 	       "-o RED   --oscope=RED      oscilloscope mode, reduce verbose output by RED\n"
 	       "-p PRIO  --priority=PRIO   priority of highest prio thread\n"
@@ -1013,12 +893,6 @@ static int interval = DEFAULT_INTERVAL;
 static int distance = -1;
 static struct bitmask *affinity_mask = NULL;
 static int smp = 0;
-
-enum {
-	AFFINITY_UNSPECIFIED,
-	AFFINITY_SPECIFIED,
-	AFFINITY_USEALL
-};
 static int setaffinity = AFFINITY_UNSPECIFIED;
 
 static int clocksources[] = {
@@ -1026,49 +900,6 @@ static int clocksources[] = {
 	CLOCK_REALTIME,
 };
 
-static unsigned int is_cpumask_zero(const struct bitmask *mask)
-{
-	return (rt_numa_bitmask_count(mask) == 0);
-}
-
-static int cpu_for_thread(int thread_num, int max_cpus)
-{
-	unsigned int m, cpu, i, num_cpus;
-	num_cpus = rt_numa_bitmask_count(affinity_mask);
-
-	m = thread_num % num_cpus;
-
-	/* there are num_cpus bits set, we want position of m'th one */
-	for (i = 0, cpu = 0; i < max_cpus; i++) {
-		if (rt_numa_bitmask_isbitset(affinity_mask, i)) {
-			if (cpu == m)
-				return i;
-			cpu++;
-		}
-	}
-	fprintf(stderr, "Bug in cpu mask handling code.\n");
-	return 0;
-}
-
-
-static void parse_cpumask(const char *option, const int max_cpus)
-{
-	affinity_mask = rt_numa_parse_cpustring(option, max_cpus);
-	if (affinity_mask) {
-		if (is_cpumask_zero(affinity_mask)) {
-			rt_bitmask_free(affinity_mask);
-			affinity_mask = NULL;
-		}
-	}
-	if (!affinity_mask)
-		display_help(1);
-
-	if (verbose) {
-		printf("%s: Using %u cpus.\n", __func__,
-			rt_numa_bitmask_count(affinity_mask));
-	}
-}
-
 static void handlepolicy(char *polname)
 {
 	if (strncasecmp(polname, "other", 5) == 0)
@@ -1125,7 +956,7 @@ enum option_values {
 };
 
 /* Process commandline options */
-static void process_options (int argc, char *argv[], int max_cpus)
+static void process_options(int argc, char *argv[], int max_cpus)
 {
 	int error = 0;
 	int option_affinity = 0;
@@ -1167,6 +998,7 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			{"spike",	     required_argument, NULL, OPT_TRIGGER },
 			{"spike-nodes",	     required_argument, NULL, OPT_TRIGGER_NODES },
 			{"threads",          optional_argument, NULL, OPT_THREADS },
+			{"tracemark",	     no_argument,	NULL, OPT_TRACEMARK },
 			{"unbuffered",       no_argument,       NULL, OPT_UNBUFFERED },
 			{"verbose",          no_argument,       NULL, OPT_VERBOSE },
 			{"dbg_cyclictest",   no_argument,       NULL, OPT_DBGCYCLIC },
@@ -1186,22 +1018,34 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			/* smp sets AFFINITY_USEALL in OPT_SMP */
 			if (smp)
 				break;
-			if (optarg != NULL) {
-				parse_cpumask(optarg, max_cpus);
+			if (numa_initialize())
+				fatal("Couldn't initialize libnuma");
+			numa = 1;
+			if (optarg) {
+				parse_cpumask(optarg, max_cpus, &affinity_mask);
 				setaffinity = AFFINITY_SPECIFIED;
-			} else if (optind<argc && atoi(argv[optind])) {
-				parse_cpumask(argv[optind], max_cpus);
+			} else if (optind < argc &&
+				   (atoi(argv[optind]) ||
+				    argv[optind][0] == '0' ||
+				    argv[optind][0] == '!')) {
+				parse_cpumask(argv[optind], max_cpus, &affinity_mask);
 				setaffinity = AFFINITY_SPECIFIED;
 			} else {
 				setaffinity = AFFINITY_USEALL;
 			}
+
+			if (setaffinity == AFFINITY_SPECIFIED && !affinity_mask)
+				display_help(1);
+			if (verbose)
+				printf("Using %u cpus.\n",
+					numa_bitmask_weight(affinity_mask));
 			break;
 		case 'A':
 		case OPT_ALIGNED:
-			aligned=1;
+			aligned = 1;
 			if (optarg != NULL)
 				offset = atoi(optarg) * 1000;
-			else if (optind<argc && atoi(argv[optind]))
+			else if (optind < argc && atoi(argv[optind]))
 				offset = atoi(argv[optind]) * 1000;
 			else
 				offset = 0;
@@ -1285,7 +1129,7 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			if (numa)
 				fatal("numa and smp options are mutually exclusive\n");
 			smp = 1;
-			num_threads = max_cpus;
+			num_threads = -1; /* update after parsing */
 			setaffinity = AFFINITY_USEALL;
 			break;
 		case 't':
@@ -1296,10 +1140,10 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			}
 			if (optarg != NULL)
 				num_threads = atoi(optarg);
-			else if (optind<argc && atoi(argv[optind]))
+			else if (optind < argc && atoi(argv[optind]))
 				num_threads = atoi(argv[optind]);
 			else
-				num_threads = max_cpus;
+				num_threads = -1; /* update after parsing */
 			break;
 		case OPT_TRIGGER:
 			trigger = atoi(optarg);
@@ -1343,18 +1187,25 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			fatal("--smi is not available on your arch\n");
 #endif
 			break;
+		case OPT_TRACEMARK:
+			trace_marker = 1; break;
 		}
 	}
 
+	if ((use_system == MODE_SYS_OFFSET) && (use_nanosleep == MODE_CYCLIC)) {
+		warn("The system option requires clock_nanosleep\n");
+		warn("and is not compatible with posix_timers\n");
+		warn("Using clock_nanosleep\n");
+		use_nanosleep = MODE_CLOCK_NANOSLEEP;
+	}
+
 	/* if smp wasn't requested, test for numa automatically */
 	if (!smp) {
-#ifdef NUMA
-		if (numa_available() != -1)
-			numa = 1;
-#else
-		warn("cyclictest was not built with the numa option\n");
-		numa = 0;
-#endif
+		if (numa_initialize())
+			fatal("Couldn't initialize libnuma");
+		numa = 1;
+		if (setaffinity == AFFINITY_UNSPECIFIED)
+			setaffinity = AFFINITY_USEALL;
 	}
 
 	if (option_affinity) {
@@ -1371,9 +1222,6 @@ static void process_options (int argc, char *argv[], int max_cpus)
 			      "on this processor\n");
 	}
 
-	if (tracelimit)
-		fileprefix = procfileprefix;
-
 	if (clocksel < 0 || clocksel > ARRAY_SIZE(clocksources))
 		error = 1;
 
@@ -1399,6 +1247,9 @@ static void process_options (int argc, char *argv[], int max_cpus)
 	if (priority < 0 || priority > 99)
 		error = 1;
 
+	if (num_threads == -1)
+		num_threads = get_available_cpus(affinity_mask);
+
 	if (priospread && priority == 0) {
 		fprintf(stderr, "defaulting realtime priority to %d\n",
 			num_threads+1);
@@ -1457,6 +1308,23 @@ static void sighand(int sig)
 		fprintf(stderr, "#---------------------------\n");
 		quiet = oldquiet;
 		return;
+	} else if (sig == SIGUSR2) {
+		int i;
+		int oldquiet = quiet;
+
+		if (rstat_fd == -1) {
+			fprintf(stderr, "ERROR: rstat_fd not valid\n");
+			return;
+		}
+		rstat_ftruncate(rstat_fd, 0);
+		quiet = 0;
+		dprintf(rstat_fd, "#---------------------------\n");
+		dprintf(rstat_fd, "# cyclictest current status:\n");
+		for (i = 0; i < num_threads; i++)
+			rstat_print_stat(parameters[i], i, 0, 0);
+		dprintf(rstat_fd, "#---------------------------\n");
+		quiet = oldquiet;
+		return;
 	}
 	shutdown = 1;
 	if (refresh_on_max)
@@ -1589,7 +1457,7 @@ static void print_stat(FILE *fp, struct thread_param *par, int index, int verbos
 				(long)(stat->avg/stat->cycles) : 0, stat->max);
 
 			if (smi)
-				fprintf(fp," SMI:%8ld", stat->smi_count);
+				fprintf(fp, " SMI:%8ld", stat->smi_count);
 
 			fprintf(fp, "\n");
 		}
@@ -1624,6 +1492,62 @@ static void print_stat(FILE *fp, struct thread_param *par, int index, int verbos
 	}
 }
 
+static void rstat_print_stat(struct thread_param *par, int index, int verbose, int quiet)
+{
+	struct thread_stat *stat = par->stats;
+	int fd = rstat_fd;
+
+	if (!verbose) {
+		if (quiet != 1) {
+			char *fmt;
+			if (use_nsecs)
+				fmt = "T:%2d (%5d) P:%2d I:%ld C:%7lu "
+				        "Min:%7ld Act:%8ld Avg:%8ld Max:%8ld";
+			else
+				fmt = "T:%2d (%5d) P:%2d I:%ld C:%7lu "
+				        "Min:%7ld Act:%5ld Avg:%5ld Max:%8ld";
+
+			dprintf(fd, fmt, index, stat->tid, par->prio,
+				par->interval, stat->cycles, stat->min,
+				stat->act, stat->cycles ?
+				(long)(stat->avg/stat->cycles) : 0, stat->max);
+
+			if (smi)
+				dprintf(fd, " SMI:%8ld", stat->smi_count);
+
+			dprintf(fd, "\n");
+		}
+	} else {
+		while (stat->cycles != stat->cyclesread) {
+			unsigned long diff_smi;
+			long diff = stat->values
+			    [stat->cyclesread & par->bufmsk];
+
+			if (smi)
+				diff_smi = stat->smis
+				[stat->cyclesread & par->bufmsk];
+
+			if (diff > stat->redmax) {
+				stat->redmax = diff;
+				stat->cycleofmax = stat->cyclesread;
+			}
+			if (++stat->reduce == oscope_reduction) {
+				if (!smi)
+					dprintf(fd, "%8d:%8lu:%8ld\n", index,
+						stat->cycleofmax, stat->redmax);
+				else
+					dprintf(fd, "%8d:%8lu:%8ld%8ld\n",
+						index, stat->cycleofmax,
+						stat->redmax, diff_smi);
+
+				stat->reduce = 0;
+				stat->redmax = 0;
+			}
+			stat->cyclesread++;
+		}
+	}
+}
+
 
 /*
  * thread that creates a named fifo and hands out run stats when someone
@@ -1688,7 +1612,8 @@ static void trigger_print()
 	struct thread_trigger *trig = head;
 	char *fmt = "T:%2d Spike:%8ld: TS: %12ld\n";
 
-	if (current == head) return;
+	if (current == head)
+		return;
 	printf("\n");
 	while (trig->next != current) {
 		fprintf(stdout, fmt,  trig->tnum, trig->diff, trig->ts);
@@ -1711,11 +1636,111 @@ static void trigger_update(struct thread_param *par, int diff, int64_t ts)
 	pthread_mutex_unlock(&trigger_lock);
 }
 
+/* Running status shared memory open */
+static int rstat_shm_open(void)
+{
+	int fd;
+	pid_t pid;
+
+	pid = getpid();
+
+	snprintf(shm_name, SHM_BUF_SIZE, "%s%d", "/cyclictest", pid);
+
+	errno = 0;
+	fd = shm_unlink(shm_name);
+
+	if ((fd == -1) && (errno != ENOENT)) {
+		fprintf(stderr, "ERROR: shm_unlink %s\n", strerror(errno));
+		return fd;
+	}
+
+	errno = 9;
+	fd = shm_open(shm_name, O_RDWR|O_CREAT|O_EXCL, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
+	if (fd == -1)
+		fprintf(stderr, "ERROR: shm_open %s\n", strerror(errno));
+
+	rstat_fd = fd;
+
+	return fd;
+}
+
+static int rstat_ftruncate(int fd, off_t len)
+{
+	int err;
+
+	errno = 0;
+	err = ftruncate(fd, len);
+	if (err)
+		fprintf(stderr, "ftruncate error %s\n", strerror(errno));
+
+	return err;
+}
+
+static void *rstat_mmap(int fd)
+{
+	void *mptr;
+
+	errno = 0;
+	mptr = mmap(0, _SC_PAGE_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, fd, 0);
+
+	if (mptr == (void*)-1)
+		fprintf(stderr, "ERROR: mmap, %s\n", strerror(errno));
+
+	return mptr;
+}
+
+static int rstat_mlock(void *mptr)
+{
+	int err;
+
+	errno = 0;
+	err = mlock(mptr, _SC_PAGE_SIZE);
+	if (err == -1)
+		fprintf(stderr, "ERROR, mlock %s\n", strerror(errno));
+
+	return err;
+}
+
+static void rstat_setup(void)
+{
+	int res;
+	void *mptr = NULL;
+
+	int sfd = rstat_shm_open();
+	if (sfd < 0)
+		goto rstat_err;
+
+	res = rstat_ftruncate(sfd, _SC_PAGE_SIZE);
+	if (res)
+		goto rstat_err1;
+
+	mptr = rstat_mmap(sfd);
+	if (mptr == MAP_FAILED)
+		goto rstat_err1;
+
+	res = rstat_mlock(mptr);
+	if (res)
+		goto rstat_err2;
+
+	return;
+
+rstat_err2:
+	munmap(mptr, _SC_PAGE_SIZE);
+rstat_err1:
+	close(sfd);
+	shm_unlink(shm_name);
+rstat_err:
+	rstat_fd = -1;
+	return;
+}
+
+
 int main(int argc, char **argv)
 {
 	sigset_t sigset;
 	int signum = SIGALRM;
 	int mode;
+	int cpu;
 	int max_cpus = sysconf(_SC_NPROCESSORS_ONLN);
 	int i, ret = -1;
 	int status;
@@ -1728,6 +1753,16 @@ int main(int argc, char **argv)
 	if (verbose)
 		printf("Max CPUs = %d\n", max_cpus);
 
+	/* Restrict the main pid to the affinity specified by the user */
+	if (affinity_mask != NULL) {
+		int res;
+
+		errno = 0;
+		res = numa_sched_setaffinity(getpid(), affinity_mask);
+		if (res != 0)
+			warn("Couldn't setaffinity in main thread: %s\n", strerror(errno));
+	}
+
 	if (trigger) {
 		int retval;
 		retval = trigger_init();
@@ -1747,7 +1782,7 @@ int main(int argc, char **argv)
 	/* use the /dev/cpu_dma_latency trick if it's there */
 	set_latency_target();
 
-	if (tracelimit)
+	if (tracelimit && trace_marker)
 		enable_trace_mark();
 
 	if (check_timer())
@@ -1767,11 +1802,10 @@ int main(int argc, char **argv)
 
 		clock = clocksources[clocksel];
 
-		if (clock_getres(clock, &res)) {
+		if (clock_getres(clock, &res))
 			warn("clock_getres failed");
-		} else {
+		else
 			reported_resolution = (NSEC_PER_SEC * res.tv_sec) + res.tv_nsec;
-		}
 
 
 		/*
@@ -1782,9 +1816,8 @@ int main(int argc, char **argv)
 		 */
 		times = 1000;
 		clock_gettime(clock, &prev);
-		for (k=0; k < times; k++) {
+		for (k=0; k < times; k++)
 			clock_gettime(clock, &now);
-		}
 
 		diff = calcdiff_ns(now, prev);
 		if (diff == 0) {
@@ -1807,9 +1840,8 @@ int main(int argc, char **argv)
 
 		time = calloc(times, sizeof(*time));
 
-		for (k=0; k < times; k++) {
+		for (k=0; k < times; k++)
 			clock_gettime(clock, &time[k]);
-		}
 
 		if (ct_debug) {
 			info("For %d consecutive calls to clock_gettime():\n", times);
@@ -1822,9 +1854,8 @@ int main(int argc, char **argv)
 			diff = calcdiff_ns(time[k], prev);
 			prev = time[k];
 
-			if (diff && (diff < min_non_zero_diff)) {
+			if (diff && (diff < min_non_zero_diff))
 				min_non_zero_diff = diff;
-			}
 
 			if (ct_debug)
 				info("%ld.%06ld  %5llu\n",
@@ -1850,15 +1881,20 @@ int main(int argc, char **argv)
 
 	}
 
+
 	mode = use_nanosleep + use_system;
 
 	sigemptyset(&sigset);
 	sigaddset(&sigset, signum);
-	sigprocmask (SIG_BLOCK, &sigset, NULL);
+	sigprocmask(SIG_BLOCK, &sigset, NULL);
 
 	signal(SIGINT, sighand);
 	signal(SIGTERM, sighand);
 	signal(SIGUSR1, sighand);
+	signal(SIGUSR2, sighand);
+
+	/* Set-up shm */
+	rstat_setup();
 
 	parameters = calloc(num_threads, sizeof(struct thread_param *));
 	if (!parameters)
@@ -1877,6 +1913,19 @@ int main(int argc, char **argv)
 		if (status != 0)
 			fatal("error from pthread_attr_init for thread %d: %s\n", i, strerror(status));
 
+		switch (setaffinity) {
+		case AFFINITY_UNSPECIFIED: cpu = -1; break;
+		case AFFINITY_SPECIFIED:
+			cpu = cpu_for_thread_sp(i, max_cpus, affinity_mask);
+			if (verbose)
+				printf("Thread %d using cpu %d.\n", i, cpu);
+			break;
+		case AFFINITY_USEALL:
+			cpu = cpu_for_thread_ua(i, max_cpus);
+			break;
+		default: cpu = -1;
+		}
+
 		node = -1;
 		if (numa) {
 			void *stack;
@@ -1884,9 +1933,9 @@ int main(int argc, char **argv)
 			size_t stksize;
 
 			/* find the memory node associated with the cpu i */
-			node = rt_numa_numa_node_of_cpu(i);
+			node = rt_numa_numa_node_of_cpu(cpu);
 
-			/* get the stack size set for for this thread */
+			/* get the stack size set for this thread */
 			if (pthread_attr_getstack(&attr, &currstk, &stksize))
 				fatal("failed to get stack size for thread %d\n", i);
 
@@ -1895,7 +1944,7 @@ int main(int argc, char **argv)
 				stksize = PTHREAD_STACK_MIN * 2;
 
 			/*  allocate memory for a stack on appropriate node */
-			stack = rt_numa_numa_alloc_onnode(stksize, node, i);
+			stack = rt_numa_numa_alloc_onnode(stksize, node, cpu);
 
 			/* touch the stack pages to pre-fault them in */
 			memset(stack, 0, stksize);
@@ -1965,20 +2014,13 @@ int main(int argc, char **argv)
 			interval += distance;
 		if (verbose)
 			printf("Thread %d Interval: %d\n", i, interval);
+
 		par->max_cycles = max_cycles;
 		par->stats = stat;
 		par->node = node;
 		par->tnum = i;
-		switch (setaffinity) {
-		case AFFINITY_UNSPECIFIED: par->cpu = -1; break;
-		case AFFINITY_SPECIFIED:
-			par->cpu = cpu_for_thread(i, max_cpus);
-			if (verbose)
-				printf("Thread %d using cpu %d.\n", i,
-					par->cpu);
-			break;
-		case AFFINITY_USEALL: par->cpu = i % max_cpus; break;
-		}
+		par->cpu = cpu;
+
 		stat->min = 1000000;
 		stat->max = 0;
 		stat->avg = 0.0;
@@ -2024,7 +2066,7 @@ int main(int argc, char **argv)
 		for (i = 0; i < num_threads; i++) {
 
 			print_stat(stdout, parameters[i], i, verbose, quiet);
-			if(max_cycles && statistics[i]->cycles >= max_cycles)
+			if (max_cycles && statistics[i]->cycles >= max_cycles)
 				allstopped++;
 		}
 
@@ -2099,10 +2141,7 @@ int main(int argc, char **argv)
 	}
  out:
 	/* close any tracer file descriptors */
-	if (tracemark_fd >= 0)
-		close(tracemark_fd);
-	if (trace_fd >= 0)
-		close(trace_fd);
+	disable_trace_mark();
 
 	/* unlock everything */
 	if (lockall)
@@ -2115,5 +2154,9 @@ int main(int argc, char **argv)
 	if (affinity_mask)
 		rt_bitmask_free(affinity_mask);
 
+	/* Remove running status shared memory file if it exists */
+	if (rstat_fd >= 0)
+		shm_unlink(shm_name);
+
 	exit(ret);
 }
