@@ -362,7 +362,37 @@ int parse_time_string(char *val)
 	return t;
 }
 
-void open_tracemark_fd(void)
+int parse_mem_string(char *str, uint64_t *val)
+{
+	char *endptr;
+	int v = strtol(str, &endptr, 10);
+
+	if (!*endptr)
+		return v;
+
+	switch (*endptr) {
+	case 'g':
+	case 'G':
+		v *= 1024;
+	case 'm':
+	case 'M':
+		v *= 1024;
+	case 'k':
+	case 'K':
+		v *= 1024;
+	case 'b':
+	case 'B':
+		break;
+	default:
+		return -1;
+	}
+
+	*val = v;
+
+	return 0;
+}
+
+static void open_tracemark_fd(void)
 {
 	char path[MAX_PATH];
 
@@ -390,16 +420,25 @@ void open_tracemark_fd(void)
 	}
 }
 
-int trace_file_exists(char *name)
+static void close_tracemark_fd(void)
 {
-       struct stat sbuf;
-       char *tracing_prefix = get_debugfileprefix();
-       char path[MAX_PATH];
-       strcat(strcpy(path, tracing_prefix), name);
-       return stat(path, &sbuf) ? 0 : 1;
+	if (tracemark_fd > 0)
+		close(tracemark_fd);
+
+	if (trace_fd > 0)
+		close(trace_fd);
 }
 
-void debugfs_prepare(void)
+static int trace_file_exists(char *name)
+{
+	struct stat sbuf;
+	char *tracing_prefix = get_debugfileprefix();
+	char path[MAX_PATH];
+	strcat(strcpy(path, tracing_prefix), name);
+	return stat(path, &sbuf) ? 0 : 1;
+}
+
+static void debugfs_prepare(void)
 {
 	if (mount_debugfs(NULL))
 		fatal("could not mount debugfs");
@@ -437,3 +476,8 @@ void enable_trace_mark(void)
 	debugfs_prepare();
 	open_tracemark_fd();
 }
+
+void disable_trace_mark(void)
+{
+	close_tracemark_fd();
+}
